#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
@author	JeeHang Lee (jeehanglee@gmail.com)
@date	20160127
@description

This is a script to retrieve the iBert playing time at each home participated in ENLITEN project.
Will support
	1. daily/weekly/in phase statistics
	2. hopefully link to the visualisation
'''

import sys
import getopt
import statistics

#custom
import mysql_util as dbutil		# db utils (custom)
import query_strings as qs		# query strings (custom)
import ibert_common as ic
import enliten_visualise as ev

import plotly					# visualisation
import plotly.graph_objs

from plotly.graph_objs import Scatter, Layout, Box, Bar
from datetime import time, datetime, date, timedelta

'''
Constants
'''
ARG_NUM = 4
CMD_HELP = 'iBertPlayingTime.py <option:daily/weekly/phase> \
		<datefrom:YYYY-MM-DD HH:MM:SS> <dateto::YYYY-MM-DD HH:MM:SS>'

FILENAME_CONVENTION = "/home/jeehang/iBert_playing_%s_from_%s_to_%s.csv"

ITEM = "%d, "

TIME = "timestamp"
TYPE = "type"

STATE_APP_CREATED = 100
STATE_APP_STARTED = 101
STATE_APP_RESTARTED = 102
STATE_APP_RESUMED = 103
STATE_APP_PAUSED = 104
STATE_APP_STOPPED = 105
STATE_APP_DESTROYED = 106

'''
Queries
'''
HOMES = "select f.home, f.access_key \
	from feedback_auth f \
	left join tablets t \
	on f.home = t.home \
	where t.deployed is not null \
	order by f.home asc"

'''
Functions
'''
#
def setItem(outItem, phase, group, mean, stdev):
	if phase < 0:
		print 'phase: out of range (which should be 0 <= phase < =5'
		return
	
	if phase < 1 or phase > 8:
		outItem[ic.quadrant[phase]] = mean
	else:
		idx = ((phase/2 + phase%2) + group - 1) % 4 
		idx = 4 if idx is 0 else idx 
		qid = idx * 2 - phase % 2
		outItem[ic.quadrant[qid]] = mean

def isInitialState(state, last_state, last_ts):
	res = False

	if (state >= STATE_APP_STARTED
		and last_state is 0
		and last_ts is ''):
		res = True
	else:
		res = False

	return res

def getDailyPlayingTime(daily_logs):
	last_state = 0
	last_ts = ''
	elapsed = 0
	daily_play = 0

	filtered_logs = filter(lambda l: l["type"] > STATE_APP_CREATED 
								and l["type"] < STATE_APP_DESTROYED, daily_logs)

	for log in filtered_logs:
		ts = log["timestamp"]
		state = log["type"]		

		if (isInitialState(state, last_state, last_ts) == True):
			last_ts = ts
			last_state = state
		elif (state == STATE_APP_STOPPED 
			and last_state >= STATE_APP_STARTED
			and last_ts is not ''):
			diff = ts - last_ts
			daily_play += diff.total_seconds()
			last_state = 0			
			last_ts = ''
			#print diff
		else:
			last_state = state

	return daily_play

def QSTART(datefrom, phase):
	start = datefrom
	offset = timedelta(days = 14)
	
	if phase > 0 and phase < 9 and (phase % 2) is 1:
		start += offset
		
	return start
	
def QEND(dateto, phase):
	end = dateto
	offset = timedelta(days = 14)
	
	if phase > 0 and phase < 9 and (phase % 2) is 0:
		end -= offset
		
	return end

def initItem(item):
	for i in ic.quadrant:
		item[i] = float('nan')

def getQuadrantStatsWeeklyBasis(house, outList):

	item = {'home':'%s', 'type':"ibert_time", 'before':'%d', 'after':'%d', 
			'wb_nvna':'%d', 'nvna':'%d', 'wb_vna':'%d', 'vna':'%d', 
			'wb_nva':'%d', 'nva':'%d', 'wb_va':'%d', 'va':'%d'}

	item['home'] = house['home']
	initItem(item)

	start = ic.PHASE_ONE['from']
	end = ic.PHASE_FOUR['to']
	records = dbutil.select(qs.PLAY_LOG % (int(house["access_key"]), start, end))
	
	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)
	
	phase = 2
	group = int(house['schedule_group'])
	list_playing_time = []
	#list_playing_time.append(int(home["home"]))
	
	while (datePhaseTo <= dateEnd):
		ws = QSTART(datePhaseFrom, phase)
		we = QEND(datePhaseTo, phase)
		phaseList = filter(lambda r: r["timestamp"].date() >= ws and 
						   r["timestamp"].date() < we,
						   records)

		dt_cur = ws
		dt_to = we
		while (dt_cur < dt_to):
			daily_logs = filter(lambda l: l["timestamp"].date() == dt_cur, phaseList)
			daily_playing_time = getDailyPlayingTime(daily_logs)
			if daily_playing_time > 0:
				list_playing_time.append(daily_playing_time)
			dt_cur += timedelta(days = 1)
		
		mean = statistics.mean(list_playing_time) / 60 if len(list_playing_time) > 0 else float('nan')
		stdev = statistics.stdev(list_playing_time) / 60 if len(list_playing_time) > 1 else 0
		setItem(item, phase, group, mean, stdev)

		phase += 1
		if (phase > 0 and (phase % 2) is 0) or (phase is 9): 
			datePhaseFrom = datePhaseTo
			datePhaseTo += timedelta(days = 21)
		del list_playing_time[:]

	print item
	outList.append(item) 
	return item

def getQuadrantStatistics(house, outList):
	item = {'home':'%s', 'type':"ibert_time", 'before':'%d', 'after':'%d',
			'nvnp':'%d', 'vnp':'%d', 'nvp':'%d', 'vp':'%d'}

	item['home'] = house['home']
	item['before'] = float('nan')
	item['after'] = float('nan')

	start = ic.PHASE_ONE['from']
	end = ic.PHASE_FOUR['to']
	records = dbutil.select(qs.PLAY_LOG % (int(house["access_key"]), start, end))
	
	dateStart = datetime.strptime(start, '%Y-%m-%d %H:%M:%S').date()
	dateEnd = datetime.strptime(end, '%Y-%m-%d %H:%M:%S').date()
	datePhaseFrom = dateStart
	datePhaseTo = datePhaseFrom + timedelta(days = 21)
	
	phase = 1
	group = int(house['schedule_group'])
	list_playing_time = []
	#list_playing_time.append(int(home["home"]))
	
	while (datePhaseTo <= dateEnd):
		phaseList = filter(lambda r: r["timestamp"].date() >= datePhaseFrom and 
						   r["timestamp"].date() < datePhaseTo,
						   records)

		dt_cur = datePhaseFrom
		dt_to = datePhaseTo
		while (dt_cur < dt_to):
			daily_logs = filter(lambda l: l["timestamp"].date() == dt_cur, phaseList)
			daily_playing_time = getDailyPlayingTime(daily_logs)
			if daily_playing_time > 0:
				list_playing_time.append(daily_playing_time)
			dt_cur += timedelta(days = 1)
		
		mean = statistics.mean(list_playing_time) / 60 if len(list_playing_time) > 0 else 0
		stdev = statistics.stdev(list_playing_time) / 60 if len(list_playing_time) > 1 else 0
		setItem(item, phase, group, mean, stdev)

		phase += 1
		datePhaseFrom = datePhaseTo
		datePhaseTo += timedelta(days = 21)
		del list_playing_time[:]

	print item
	outList.append(item) 
	return item
